/* Libvisual - The audio visualisation framework.
 * 
 * Copyright (C) 2004, 2005, 2006 Dennis Smit <ds@nerds-incorporated.org>
 *
 * Authors: Dennis Smit <ds@nerds-incorporated.org>
 *
 * $Id: libvisual.h,v 1.23 2006/01/22 13:23:36 synap Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef LV_LIBVISUAL_H
#define LV_LIBVISUAL_H

#include <libvisual/lvconfig.h>

#include <libvisual/lv_bits.h>
#include <libvisual/lv_time.h>
#include <libvisual/lv_color.h>
#include <libvisual/lv_param.h>
#include <libvisual/lv_config.h>
#include <libvisual/lv_cpu.h>
#include <libvisual/lv_bin.h>
#include <libvisual/lv_event.h>
#include <libvisual/lv_keysym.h>
#include <libvisual/lv_actor.h>
#include <libvisual/lv_input.h>
#include <libvisual/lv_audio.h>
#include <libvisual/lv_fourier.h>
#include <libvisual/lv_list.h>
#include <libvisual/lv_palette.h>
#include <libvisual/lv_plugin.h>
#include <libvisual/lv_video.h>
#include <libvisual/lv_libvisual.h>
#include <libvisual/lv_songinfo.h>
#include <libvisual/lv_morph.h>
#include <libvisual/lv_transform.h>
#include <libvisual/lv_bmp.h>
#include <libvisual/lv_log.h>
#include <libvisual/lv_random.h>
#include <libvisual/lv_ui.h>
#include <libvisual/lv_collection.h>
#include <libvisual/lv_hashmap.h>
#include <libvisual/lv_cache.h>
#include <libvisual/lv_ringbuffer.h>
#include <libvisual/lv_rectangle.h>
#include <libvisual/lv_error.h>
#include <libvisual/lv_thread.h>
#include <libvisual/lv_object.h>
#include <libvisual/lv_types.h>
#include <libvisual/lv_utils.h>
#include <libvisual/lv_gl.h>
#include <libvisual/lv_math.h>
#include <libvisual/lv_os.h>
#include <libvisual/lv_common.h>

#endif /* LV_LIBVISUAL_H */
